@@ -345,11 +345,21 @@ class CORE_EXPORT QgsGeometry
      */
     bool isGeosEqual( const QgsGeometry & ) const;
 
+    //! Validity check flags
+    enum ValidityFlag
+    {
+      FlagAllowSelfTouchingHoles = 1 << 0, //!< Indicates that self-touching holes are permitted. OGC validity states that self-touching holes are NOT permitted, whilst other vendor validity checks (e.g. ESRI) permit self-touching holes.
+    };
+    Q_DECLARE_FLAGS( ValidityFlags, ValidityFlag )
+
     /**
-     * Checks validity of the geometry using GEOS
+     * Checks validity of the geometry using GEOS.
+     *
+     * The \a flags parameter indicates optional flags which control the type of validity checking performed.
+     *
      * \since QGIS 1.5
      */
-    bool isGeosValid() const;
+    bool isGeosValid( QgsGeometry::ValidityFlags flags = nullptr ) const;
 
     /**
      * Determines whether the geometry is simple (according to OGC definition),
@@ -1841,6 +1851,11 @@ class CORE_EXPORT QgsGeometry
         % End
 #endif
 
+        bool operator==( const QgsGeometry::Error &other ) const
+        {
+          return other.mMessage == mMessage && other.mHasLocation == mHasLocation && other.mLocation == mLocation;
+        }
+
       private:
         QString mMessage;
         QgsPointXY mLocation;
@@ -1860,10 +1875,12 @@ class CORE_EXPORT QgsGeometry
     /**
      * Validates geometry and produces a list of geometry errors.
      * The \a method argument dictates which validator to utilize.
-     * \note Available in Python bindings since QGIS 1.6
+     *
+     * The \a flags parameter indicates optional flags which control the type of validity checking performed.
+     *
      * \since QGIS 1.5
      */
-    void validateGeometry( QVector<QgsGeometry::Error> &errors SIP_OUT, ValidationMethod method = ValidatorQgisInternal ) const;
+    void validateGeometry( QVector<QgsGeometry::Error> &errors SIP_OUT, ValidationMethod method = ValidatorQgisInternal, QgsGeometry::ValidityFlags flags = nullptr ) const;
 
     /**
      * Compute the unary union on a list of \a geometries. May be faster than an iterative union on a set of geometries.
@@ -2270,6 +2287,7 @@ class CORE_EXPORT QgsGeometry
 }; // class QgsGeometry
 
 Q_DECLARE_METATYPE( QgsGeometry )
+Q_DECLARE_OPERATORS_FOR_FLAGS( QgsGeometry::ValidityFlags )
 
 //! Writes the geometry to stream out. QGIS version compatibility is not guaranteed.
 CORE_EXPORT QDataStream &operator<<( QDataStream &out, const QgsGeometry &geometry );
