@@ -23,7 +23,7 @@ __copyright__ = '(C) 2014, Radoslaw Guzinski'
 
 # This will get replaced with a git SHA1 when you do a git archive
 
-__revision__ = 'f6ddc62fdb8ae589d3827e84d0c25bf669931040'
+__revision__ = '0c5774c0685d4b6f3e92a086234455708d1cd427'
 
 import os
 
@@ -39,6 +39,7 @@ from qgis.core import (QgsProcessingAlgorithm,
                        QgsProcessingParameterBoolean,
                        QgsProcessingParameterRasterDestination,
                        QgsProcessingParameterCrs,
+                       QgsProcessingParameterString,
                        QgsProcessingOutputLayerDefinition,
                        QgsProcessingUtils)
 from processing.algs.gdal.GdalAlgorithm import GdalAlgorithm
@@ -57,6 +58,7 @@ class buildvrt(GdalAlgorithm):
     ADD_ALPHA = 'ADD_ALPHA'
     ASSIGN_CRS = 'ASSIGN_CRS'
     RESAMPLING = 'RESAMPLING'
+    SRC_NODATA = 'SRC_NODATA'
 
     RESOLUTION_OPTIONS = ['average', 'highest', 'lowest']
     RESAMPLING_OPTIONS = ['nearest', 'bilinear', 'cubic', 'cubicspline', 'lanczos', 'average', 'mode']
@@ -82,38 +84,45 @@ class buildvrt(GdalAlgorithm):
                 return 'vrt'
 
         self.addParameter(QgsProcessingParameterMultipleLayers(self.INPUT,
-                                                               QCoreApplication.translate("ParameterVrtDestination", 'Input layers'),
+                                                               self.tr('Input layers'),
                                                                QgsProcessing.TypeRaster))
         self.addParameter(QgsProcessingParameterEnum(self.RESOLUTION,
-                                                     QCoreApplication.translate("ParameterVrtDestination", 'Resolution'),
+                                                     self.tr('Resolution'),
                                                      options=self.RESOLUTION_OPTIONS,
                                                      defaultValue=0))
         self.addParameter(QgsProcessingParameterBoolean(self.SEPARATE,
-                                                        QCoreApplication.translate("ParameterVrtDestination", 'Place each input file into a separate band'),
+                                                        self.tr('Place each input file into a separate band'),
                                                         defaultValue=True))
         self.addParameter(QgsProcessingParameterBoolean(self.PROJ_DIFFERENCE,
-                                                        QCoreApplication.translate("ParameterVrtDestination", 'Allow projection difference'),
+                                                        self.tr('Allow projection difference'),
                                                         defaultValue=False))
 
         add_alpha_param = QgsProcessingParameterBoolean(self.ADD_ALPHA,
-                                                        QCoreApplication.translate("ParameterVrtDestination", 'Add alpha mask band to VRT when source raster has none'),
+                                                        self.tr('Add alpha mask band to VRT when source raster has none'),
                                                         defaultValue=False)
         add_alpha_param.setFlags(add_alpha_param.flags() | QgsProcessingParameterDefinition.FlagAdvanced)
         self.addParameter(add_alpha_param)
 
         assign_crs = QgsProcessingParameterCrs(self.ASSIGN_CRS,
-                                               QCoreApplication.translate("ParameterVrtDestination", 'Override projection for the output file'),
+                                               self.tr('Override projection for the output file'),
                                                defaultValue=None, optional=True)
         assign_crs.setFlags(assign_crs.flags() | QgsProcessingParameterDefinition.FlagAdvanced)
         self.addParameter(assign_crs)
 
         resampling = QgsProcessingParameterEnum(self.RESAMPLING,
-                                                QCoreApplication.translate("ParameterVrtDestination", 'Resampling algorithm'),
+                                                self.tr('Resampling algorithm'),
                                                 options=self.RESAMPLING_OPTIONS,
                                                 defaultValue=0)
         resampling.setFlags(resampling.flags() | QgsProcessingParameterDefinition.FlagAdvanced)
         self.addParameter(resampling)
 
+        src_nodata_param = QgsProcessingParameterString(self.SRC_NODATA,
+                                                        self.tr('Nodata value(s) for input bands (space separated)'),
+                                                        defaultValue=None,
+                                                        optional=True)
+        src_nodata_param.setFlags(src_nodata_param.flags() | QgsProcessingParameterDefinition.FlagAdvanced)
+        self.addParameter(src_nodata_param)
+
         self.addParameter(ParameterVrtDestination(self.OUTPUT, QCoreApplication.translate("ParameterVrtDestination", 'Virtual')))
 
     def name(self):
@@ -151,6 +160,10 @@ class buildvrt(GdalAlgorithm):
         arguments.append('-r')
         arguments.append(self.RESAMPLING_OPTIONS[self.parameterAsEnum(parameters, self.RESAMPLING, context)])
 
+        if self.SRC_NODATA in parameters and parameters[self.SRC_NODATA] not in (None, ''):
+            nodata = self.parameterAsString(parameters, self.SRC_NODATA, context)
+            arguments.append('-srcnodata "{}"'.format(nodata))
+
         # Always write input files to a text file in case there are many of them and the
         # length of the command will be longer then allowed in command prompt
         list_file = GdalUtils.writeLayerParameterToTextFile(filename='buildvrtInputFiles.txt', alg=self, parameters=parameters, parameter_name=self.INPUT, context=context, executing=executing, quote=False)
@@ -158,6 +171,7 @@ class buildvrt(GdalAlgorithm):
         arguments.append(list_file)
 
         out = self.parameterAsOutputLayer(parameters, self.OUTPUT, context)
+        self.setOutputValue(self.OUTPUT, out)
         arguments.append(out)
 
         return [self.commandName(), GdalUtils.escapeAndJoin(arguments)]
