@@ -23,14 +23,15 @@ __copyright__ = '(C) 2010, Michael Minn'
 
 # This will get replaced with a git SHA1 when you do a git archive
 
-__revision__ = 'f6ddc62fdb8ae589d3827e84d0c25bf669931040'
+__revision__ = '0c5774c0685d4b6f3e92a086234455708d1cd427'
 
 from qgis.core import (QgsFeatureRequest,
                        QgsProcessingException,
                        QgsFeatureSink,
                        QgsSpatialIndex,
                        QgsProcessingParameterFeatureSource,
-                       QgsProcessingParameterFeatureSink)
+                       QgsProcessingParameterFeatureSink,
+                       QgsProcessingOutputNumber)
 from processing.algs.qgis.QgisAlgorithm import QgisAlgorithm
 
 
@@ -38,6 +39,8 @@ class DeleteDuplicateGeometries(QgisAlgorithm):
 
     INPUT = 'INPUT'
     OUTPUT = 'OUTPUT'
+    RETAINED_COUNT = 'RETAINED_COUNT'
+    DUPLICATE_COUNT = 'DUPLICATE_COUNT'
 
     def group(self):
         return self.tr('Vector general')
@@ -45,6 +48,9 @@ class DeleteDuplicateGeometries(QgisAlgorithm):
     def groupId(self):
         return 'vectorgeneral'
 
+    def tags(self):
+        return self.tr('drop,remove,same,points,coincident,overlapping,filter').split(',')
+
     def __init__(self):
         super().__init__()
 
@@ -53,6 +59,9 @@ class DeleteDuplicateGeometries(QgisAlgorithm):
                                                               self.tr('Input layer')))
         self.addParameter(QgsProcessingParameterFeatureSink(self.OUTPUT, self.tr('Cleaned')))
 
+        self.addOutput(QgsProcessingOutputNumber(self.RETAINED_COUNT, self.tr('Count of retained records')))
+        self.addOutput(QgsProcessingOutputNumber(self.DUPLICATE_COUNT, self.tr('Count of discarded duplicate records')))
+
     def name(self):
         return 'deleteduplicategeometries'
 
@@ -92,6 +101,7 @@ class DeleteDuplicateGeometries(QgisAlgorithm):
         unique_features = dict(geoms)
 
         current = 0
+        removed = 0
         for feature_id, geometry in geoms.items():
             if feedback.isCanceled():
                 break
@@ -113,6 +123,7 @@ class DeleteDuplicateGeometries(QgisAlgorithm):
                 if geometry.isGeosEqual(geoms[candidate_id]):
                     # candidate is a duplicate of feature
                     del unique_features[candidate_id]
+                    removed += 1
 
             current += 1
             feedback.setProgress(int(0.80 * current * total) + 10)  # takes about 80% of time
@@ -135,4 +146,7 @@ class DeleteDuplicateGeometries(QgisAlgorithm):
 
             feedback.setProgress(int(0.10 * current * total) + 90) # takes about 10% of time
 
-        return {self.OUTPUT: dest_id}
+        feedback.pushInfo(self.tr('{} duplicate features removed'.format(removed)))
+        return {self.OUTPUT: dest_id,
+                self.DUPLICATE_COUNT: removed,
+                self.RETAINED_COUNT: len(output_feature_ids)}
