@@ -24,7 +24,7 @@ __copyright__ = '(C) 2012, Victor Olaya'
 
 # This will get replaced with a git SHA1 when you do a git archive
 
-__revision__ = '9b176802e5512bc310765d8ebe4e44b5f4663453'
+__revision__ = '0c5774c0685d4b6f3e92a086234455708d1cd427'
 
 import os
 import warnings
@@ -83,7 +83,7 @@ class MultipleInputPanel(BASE, WIDGET):
         selectedoptions = []
         selected = [self.options[i] if isinstance(i, int) else i for i in self.selectedoptions]
         for sel in selected:
-            if isinstance(sel, int):
+            if not isinstance(sel, int):
                 try:
                     idx = options.index(sel)
                     selectedoptions.append(idx)
