@@ -23,7 +23,7 @@ __copyright__ = '(C) 2012, Alexander Bruy'
 
 # This will get replaced with a git SHA1 when you do a git archive
 
-__revision__ = '1edf372fb80f2a35c77b128dc143c0f3c97cd8f3'
+__revision__ = '0c5774c0685d4b6f3e92a086234455708d1cd427'
 
 import os
 import codecs
@@ -44,6 +44,7 @@ from qgis.core import (QgsApplication,
                        QgsProcessingAlgorithm,
                        QgsProcessingFeatureBasedAlgorithm)
 from qgis.utils import iface, OverrideCursor
+from qgis.processing import alg as algfactory
 
 from processing.gui.AlgorithmDialog import AlgorithmDialog
 from processing.script import ScriptUtils
@@ -68,7 +69,9 @@ class ScriptEditorDialog(BASE, WIDGET):
         self.editor.initLexer()
         self.searchWidget.setVisible(False)
 
-        self.toolBar.setIconSize(iface.iconSize())
+        if iface is not None:
+            self.toolBar.setIconSize(iface.iconSize())
+            self.setStyleSheet(iface.mainWindow().styleSheet())
 
         self.actionOpenScript.setIcon(
             QgsApplication.getThemeIcon('/mActionScriptOpen.svg'))
@@ -119,7 +122,6 @@ class ScriptEditorDialog(BASE, WIDGET):
         if filePath is not None:
             self._loadFile(filePath)
 
-        self.needUpdate = False
         self.setHasChanged(False)
 
     def update_dialog_title(self):
@@ -148,21 +150,15 @@ class ScriptEditorDialog(BASE, WIDGET):
                 QMessageBox.Save | QMessageBox.Cancel | QMessageBox.Discard, QMessageBox.Cancel)
 
             if ret == QMessageBox.Save:
-                self.updateProvider()
                 self.saveScript(False)
                 event.accept()
             elif ret == QMessageBox.Discard:
-                self.updateProvider()
                 event.accept()
             else:
                 event.ignore()
         else:
             event.accept()
 
-    def updateProvider(self):
-        if self.needUpdate:
-            QgsApplication.processingRegistry().providerById("script").refreshAlgorithms()
-
     def openScript(self):
         if self.hasChanged:
             ret = QMessageBox.warning(self,
@@ -176,7 +172,7 @@ class ScriptEditorDialog(BASE, WIDGET):
         fileName, _ = QFileDialog.getOpenFileName(self,
                                                   self.tr("Open script"),
                                                   scriptDir,
-                                                  self.tr("Script files (*.py *.PY)"))
+                                                  self.tr("Processing scripts (*.py *.PY)"))
 
         if fileName == "":
             return
@@ -197,7 +193,7 @@ class ScriptEditorDialog(BASE, WIDGET):
             newPath, _ = QFileDialog.getSaveFileName(self,
                                                      self.tr("Save script"),
                                                      scriptDir,
-                                                     self.tr("Script files (*.py *.PY)"))
+                                                     self.tr("Processing scripts (*.py *.PY)"))
 
             if newPath:
                 if not newPath.lower().endswith(".py"):
@@ -216,18 +212,20 @@ class ScriptEditorDialog(BASE, WIDGET):
                                     self.tr("Unable to save edits:\n{}").format(str(e))
                                     )
                 return
-            self.needUpdate = True
+
             self.setHasChanged(False)
 
+        QgsApplication.processingRegistry().providerById("script").refreshAlgorithms()
+
     def setHasChanged(self, hasChanged):
         self.hasChanged = hasChanged
         self.actionSaveScript.setEnabled(hasChanged)
         self.update_dialog_title()
 
     def runAlgorithm(self):
-        d = {}
+        _locals = {}
         try:
-            exec(self.editor.text(), d)
+            exec(self.editor.text(), _locals)
         except Exception as e:
             error = QgsError(traceback.format_exc(), "Processing")
             QgsErrorDialog.show(error,
@@ -236,10 +234,13 @@ class ScriptEditorDialog(BASE, WIDGET):
             return
 
         alg = None
-        for k, v in d.items():
-            if inspect.isclass(v) and issubclass(v, (QgsProcessingAlgorithm, QgsProcessingFeatureBasedAlgorithm)) and v.__name__ not in ("QgsProcessingAlgorithm", "QgsProcessingFeatureBasedAlgorithm"):
-                alg = v()
-                break
+        try:
+            alg = algfactory.instances.pop().createInstance()
+        except IndexError:
+            for name, attr in _locals.items():
+                if inspect.isclass(attr) and issubclass(attr, (QgsProcessingAlgorithm, QgsProcessingFeatureBasedAlgorithm)) and attr.__name__ not in ("QgsProcessingAlgorithm", "QgsProcessingFeatureBasedAlgorithm"):
+                    alg = attr()
+                    break
 
         if alg is None:
             QMessageBox.warning(self,
@@ -251,9 +252,9 @@ class ScriptEditorDialog(BASE, WIDGET):
         alg.setProvider(QgsApplication.processingRegistry().providerById("script"))
         alg.initAlgorithm()
 
-        dlg = alg.createCustomParametersWidget(self)
+        dlg = alg.createCustomParametersWidget(iface.mainWindow())
         if not dlg:
-            dlg = AlgorithmDialog(alg)
+            dlg = AlgorithmDialog(alg, parent=iface.mainWindow())
 
         canvas = iface.mapCanvas()
         prevMapTool = canvas.mapTool()
